Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Data
Imports C1.Win.C1FlexGrid


Public Class FlexFreezeRight

    ' FlexFreezeRight
    ' 
    ' This is a control (derived from Panel) that contains a FlexGrid. The
    ' contained FlexGrid is positioned with a negative X coordinate within 
    ' the panel so that the fixed columns are not visible.
    ' 
    ' The contained grid is also sized and scrolled via code so that the 
    ' right 'n' columns are visible.
    ' 
    ' The contained grid is bound to a main FlexGrid and therefore shares 
    ' the underlying grid model (data, layout, styles, selection, etc).
    ' 
    ' The whole control (panel and contained grid) is attached to the main 
    ' FlexGrid and positioned at its right. This way, the control always
    ' shows the right 'n' columns of the main grid.

    Friend Class FlexFreezeRight
        Inherits Panel
        Dim WithEvents _frz As C1.Win.C1FlexGrid.C1FlexGrid
        Dim WithEvents _main As C1.Win.C1FlexGrid.C1FlexGrid
        Dim _cols As Integer

        Friend Sub New(ByVal main As C1.Win.C1FlexGrid.C1FlexGrid, ByVal cols As Integer)
            MyBase.New()

            ' save info we'll need later
            _main = main
            _cols = cols

            ' create child grid with frozen right columns
            _frz = New C1FlexGrid()
            _frz.ScrollBars = ScrollBars.None
            _frz.BorderStyle = Util.BaseControls.BorderStyleEnum.None
            _frz.DrawMode = DrawModeEnum.OwnerDraw
            Controls.Add(_frz)

            ' bind frozen grid to main grid(share data, layout, styles,selection)
            _frz.DataSource = main

            ' add this to the main grid
            main.Controls.Add(Me)

            'initialize 
            AdjustLayout()

        End Sub

        Friend Sub AdjustLayout()

            ' get index of the right (frozen) columns
            Dim cols As ColumnCollection = _main.Cols
            Dim c2 As Integer = cols.Count - 1
            Dim c1 As Integer = c2 - _cols + 1
            If c1 <= cols.Fixed Then c1 = cols.Fixed

            ' hide control if the main grid is big enough
            Visible = (c1 <= c2)
            If Not Visible Then Return

            ' set control size and location
            Dim rc As Rectangle = _main.ClientRectangle
            If cols(c2).Right <= rc.Right Then
                Visible = False
                Return
            End If

            Dim wid As Integer = cols(c2).Right - cols(c1).Left + 1
            rc.X = rc.Right - wid
            rc.Width = wid
            Bounds = rc

            ' adjust child grid to hide fixed cols
            rc = ClientRectangle
            wid = cols(cols.Fixed + cols.Frozen).Left
            rc.X -= wid
            rc.Width += wid
            _frz.Bounds = rc

            ' synchronize scroll position
            SynchScrollPosition(False)

        End Sub

        Friend Sub SynchScrollPosition(ByVal synchMain As Boolean)
            Dim pos As Point

            ' optionally synchronize Y scroll position on the main grid
            If synchMain Then
                pos = _frz.ScrollPosition
                pos.X = _main.ScrollPosition.X
                _main.ScrollPosition = pos
            End If

            ' synchronize Y scroll position of the frozen grid with main
            pos = _main.ScrollPosition
            pos.X = -64000
            _frz.ScrollPosition = pos

            ' update everything right away
            _main.Update()
            _frz.Update()

        End Sub

        ' make normal and scrollable areas look different
        ' (when painting and when editing)
        Private Sub _frz_OwnerDrawCell(ByVal sender As Object, ByVal e As C1.Win.C1FlexGrid.OwnerDrawCellEventArgs) Handles _frz.OwnerDrawCell
            If e.Style.BackColor.Equals(_frz.Styles.Normal.BackColor) Then
                e.Graphics.FillRectangle(Brushes.Beige, e.Bounds)
                e.DrawCell(DrawCellFlags.Border Or DrawCellFlags.Content)
            End If
        End Sub

        Private Sub _frz_SetupEditor(ByVal sender As Object, ByVal e As C1.Win.C1FlexGrid.RowColEventArgs) Handles _frz.SetupEditor
            _frz.BackColor = Color.Beige
        End Sub

        ' adjust layout after resizing the main grid (our container)
        Private Sub _main_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles _main.Resize, _main.VisibleChanged
            AdjustLayout()
        End Sub

        ' synchronize scrolling
        Private Sub _main_AfterScroll(ByVal sender As Object, ByVal e As C1.Win.C1FlexGrid.RangeEventArgs) Handles _main.AfterScroll
            SynchScrollPosition(False)
        End Sub

        Private Sub _frz_AfterScroll(ByVal sender As Object, ByVal e As C1.Win.C1FlexGrid.RangeEventArgs) Handles _frz.AfterScroll

            ' if the frozen grid has the focus and the user selects non-frozen
            ' cells, scroll main grid up.
            If _frz.Focused Then
                Dim col As Integer = Math.Min(_frz.Col, _frz.ColSel)
                Dim rig As Integer = _main.Cols(col).Right + _main.ScrollPosition.X
                If rig > Left Then
                    ' unless this is a mouse selection, give focus to main grid
                    If Not _frz.Capture Then _main.Focus()
                    _main.LeftCol = _main.Col
                End If
            End If
            SynchScrollPosition(True)

        End Sub

        ' scroll while selecting 
        ' if the user extends the selection down on the main grid, scroll it down
        Private Sub _main_SelChange(ByVal sender As Object, ByVal e As System.EventArgs) Handles _main.SelChange

            ' only when the main grid has the focus!
            If Not _main.Focused Then Return

            ' calculate selection right (accounting for scrolling)
            Dim col As Integer = Math.Max(_main.Col, _main.ColSel)
            Dim rig As Integer = _main.Cols(col).Right + _main.ScrollPosition.X

            'try scrolling if past the right of the frozen area
            If rig > Left Then _main.LeftCol += 1

        End Sub

        ' adjust layout after massive changes (resize rows/cols, sort,etc)
        Private Sub _main_GridChanged(ByVal sender As Object, ByVal e As C1.Win.C1FlexGrid.GridChangedEventArgs) Handles _main.GridChanged, _frz.GridChanged

            Select Case e.GridChangedType
                Case GridChangedTypeEnum.GridChanged, GridChangedTypeEnum.LayoutChanged
                    AdjustLayout()
            End Select
        End Sub
    End Class
End Class
